=== modified file 'TODO'
--- TODO	2011-11-09 17:16:03 +0000
+++ TODO	2011-12-21 00:29:33 +0000
@@ -40,10 +40,10 @@
 ** TODO [#B] Use openat()
 
 * mandos (server)
+** TODO Document why we ignore sigint
 ** TODO [#B] Log level							  :BUGS:
+** TODO [#A] Save state to new file and move instead of overwrite
   
-*** TODO /etc/mandos/clients.d/*.conf
-    Watch this directory and add/remove/update clients?
 ** TODO [#C] config for TXT record
 ** TODO Log level option
    syslogger.setLevel(logging.WARNING)

=== modified file 'mandos'
--- mandos	2011-11-27 19:37:54 +0000
+++ mandos	2011-12-21 00:29:33 +0000
@@ -85,7 +85,6 @@
     except ImportError:
         SO_BINDTODEVICE = None
 
-
 version = "1.4.1"
 stored_state_file = "clients.pickle"
 
@@ -128,11 +127,12 @@
     logger.setLevel(level)
 
 
-class CryptoError(Exception):
+class PGPError(Exception):
+    """Exception if encryption/decryption fails"""
     pass
 
 
-class Crypto(object):
+class PGPEngine(object):
     """A simple class for OpenPGP symmetric encryption & decryption"""
     def __init__(self):
         self.gnupg = GnuPGInterface.GnuPG()
@@ -184,7 +184,7 @@
                     ciphertext = f.read()
                 proc.wait()
             except IOError as e:
-                raise CryptoError(e)
+                raise PGPError(e)
         self.gnupg.passphrase = None
         return ciphertext
     
@@ -201,7 +201,7 @@
                     decrypted_plaintext = f.read()
                 proc.wait()
             except IOError as e:
-                raise CryptoError(e)
+                raise PGPError(e)
         self.gnupg.passphrase = None
         return decrypted_plaintext
 
@@ -411,7 +411,6 @@
     interval:   datetime.timedelta(); How often to start a new checker
     last_approval_request: datetime.datetime(); (UTC) or None
     last_checked_ok: datetime.datetime(); (UTC) or None
-
     last_checker_status: integer between 0 and 255 reflecting exit
                          status of last checker. -1 reflects crashed
                          checker, or None.
@@ -431,6 +430,16 @@
                           "created", "enabled", "fingerprint",
                           "host", "interval", "last_checked_ok",
                           "last_enabled", "name", "timeout")
+    client_defaults = { "timeout": "5m",
+                        "extended_timeout": "15m",
+                        "interval": "2m",
+                        "checker": "fping -q -- %%(host)s",
+                        "host": "",
+                        "approval_delay": "0s",
+                        "approval_duration": "1s",
+                        "approved_by_default": "True",
+                        "enabled": "True",
+                        }
     
     def timeout_milliseconds(self):
         "Return the 'timeout' attribute in milliseconds"
@@ -446,63 +455,82 @@
     
     def approval_delay_milliseconds(self):
         return timedelta_to_milliseconds(self.approval_delay)
-    
-    def __init__(self, name = None, config=None):
+
+    @staticmethod
+    def config_parser(config):
+        """ Construct a new dict of client settings of this form:
+        { client_name: {setting_name: value, ...}, ...}
+        with exceptions for any special settings as defined above"""
+        settings = {}
+        for client_name in config.sections():
+            section = dict(config.items(client_name))
+            client = settings[client_name] = {}
+            
+            client["host"] = section["host"]
+            # Reformat values from string types to Python types
+            client["approved_by_default"] = config.getboolean(
+                client_name, "approved_by_default")
+            client["enabled"] = config.getboolean(client_name, "enabled")
+            
+            client["fingerprint"] = (section["fingerprint"].upper()
+                                     .replace(" ", ""))
+            if "secret" in section:
+                client["secret"] = section["secret"].decode("base64")
+            elif "secfile" in section:
+                with open(os.path.expanduser(os.path.expandvars
+                                             (section["secfile"])),
+                          "rb") as secfile:
+                    client["secret"] = secfile.read()
+            else:
+                raise TypeError("No secret or secfile for section %s"
+                                % section)
+            client["timeout"] = string_to_delta(section["timeout"])
+            client["extended_timeout"] = string_to_delta(
+                section["extended_timeout"])
+            client["interval"] = string_to_delta(section["interval"])
+            client["approval_delay"] = string_to_delta(
+                section["approval_delay"])
+            client["approval_duration"] = string_to_delta(
+                section["approval_duration"])
+            client["checker_command"] = section["checker"]
+            client["last_approval_request"] = None
+            client["last_checked_ok"] = None
+            client["last_checker_status"] = None
+            if client["enabled"]:
+                client["last_enabled"] = datetime.datetime.utcnow()
+                client["expires"] = (datetime.datetime.utcnow()
+                                     + client["timeout"])
+            else:
+                client["last_enabled"] = None
+                client["expires"] = None
+
+        return settings
+        
+        
+    def __init__(self, settings, name = None):
         """Note: the 'checker' key in 'config' sets the
         'checker_command' attribute and *not* the 'checker'
         attribute."""
         self.name = name
-        if config is None:
-            config = {}
+        # adding all client settings
+        for setting, value in settings.iteritems():
+            setattr(self, setting, value)
+        
         logger.debug("Creating client %r", self.name)
         # Uppercase and remove spaces from fingerprint for later
         # comparison purposes with return value from the fingerprint()
         # function
-        self.fingerprint = (config["fingerprint"].upper()
-                            .replace(" ", ""))
         logger.debug("  Fingerprint: %s", self.fingerprint)
-        if "secret" in config:
-            self.secret = config["secret"].decode("base64")
-        elif "secfile" in config:
-            with open(os.path.expanduser(os.path.expandvars
-                                         (config["secfile"])),
-                      "rb") as secfile:
-                self.secret = secfile.read()
-        else:
-            raise TypeError("No secret or secfile for client %s"
-                            % self.name)
-        self.host = config.get("host", "")
-        self.created = datetime.datetime.utcnow()
-        self.enabled = config.get("enabled", True)
-        self.last_approval_request = None
-        if self.enabled:
-            self.last_enabled = datetime.datetime.utcnow()
-        else:
-            self.last_enabled = None
-        self.last_checked_ok = None
-        self.last_checker_status = None
-        self.timeout = string_to_delta(config["timeout"])
-        self.extended_timeout = string_to_delta(config
-                                                ["extended_timeout"])
-        self.interval = string_to_delta(config["interval"])
+        self.created = settings.get("created", datetime.datetime.utcnow())
+
+        # attributes specific for this server instance
         self.checker = None
         self.checker_initiator_tag = None
         self.disable_initiator_tag = None
-        if self.enabled:
-            self.expires = datetime.datetime.utcnow() + self.timeout
-        else:
-            self.expires = None
         self.checker_callback_tag = None
-        self.checker_command = config["checker"]
         self.current_checker_command = None
         self.approved = None
-        self.approved_by_default = config.get("approved_by_default",
-                                              True)
         self.approvals_pending = 0
-        self.approval_delay = string_to_delta(
-            config["approval_delay"])
-        self.approval_duration = string_to_delta(
-            config["approval_duration"])
         self.changedstate = (multiprocessing_manager
                              .Condition(multiprocessing_manager
                                         .Lock()))
@@ -575,7 +603,7 @@
         self.checker_callback_tag = None
         self.checker = None
         if os.WIFEXITED(condition):
-            self.last_checker_status =  os.WEXITSTATUS(condition)
+            self.last_checker_status = os.WEXITSTATUS(condition)
             if self.last_checker_status == 0:
                 logger.info("Checker for %(name)s succeeded",
                             vars(self))
@@ -1011,6 +1039,7 @@
     def __init__(self, bus = None, *args, **kwargs):
         self.bus = bus
         Client.__init__(self, *args, **kwargs)
+        self._approvals_pending = 0
         
         self._approvals_pending = 0
         # Only now, when this client is initialized, can it show up on
@@ -1068,21 +1097,21 @@
         datetime_to_dbus, "LastApprovalRequest")
     approved_by_default = notifychangeproperty(dbus.Boolean,
                                                "ApprovedByDefault")
-    approval_delay = notifychangeproperty(dbus.UInt16,
+    approval_delay = notifychangeproperty(dbus.UInt64,
                                           "ApprovalDelay",
                                           type_func =
                                           timedelta_to_milliseconds)
     approval_duration = notifychangeproperty(
-        dbus.UInt16, "ApprovalDuration",
+        dbus.UInt64, "ApprovalDuration",
         type_func = timedelta_to_milliseconds)
     host = notifychangeproperty(dbus.String, "Host")
-    timeout = notifychangeproperty(dbus.UInt16, "Timeout",
+    timeout = notifychangeproperty(dbus.UInt64, "Timeout",
                                    type_func =
                                    timedelta_to_milliseconds)
     extended_timeout = notifychangeproperty(
-        dbus.UInt16, "ExtendedTimeout",
+        dbus.UInt64, "ExtendedTimeout",
         type_func = timedelta_to_milliseconds)
-    interval = notifychangeproperty(dbus.UInt16,
+    interval = notifychangeproperty(dbus.UInt64,
                                     "Interval",
                                     type_func =
                                     timedelta_to_milliseconds)
@@ -1279,7 +1308,7 @@
     def Host_dbus_property(self, value=None):
         if value is None:       # get
             return dbus.String(self.host)
-        self.host = value
+        self.host = unicode(value)
     
     # Created - property
     @dbus_service_property(_interface, signature="s", access="read")
@@ -1379,7 +1408,7 @@
     def Checker_dbus_property(self, value=None):
         if value is None:       # get
             return dbus.String(self.checker_command)
-        self.checker_command = value
+        self.checker_command = unicode(value)
     
     # CheckerRunning - property
     @dbus_service_property(_interface, signature="b",
@@ -1664,7 +1693,7 @@
     def sub_process_main(self, request, address):
         try:
             self.finish_request(request, address)
-        except:
+        except Exception:
             self.handle_error(request, address)
         self.close_request(request)
     
@@ -2064,15 +2093,7 @@
                                 % server_settings["servicename"]))
     
     # Parse config file with clients
-    client_defaults = { "timeout": "5m",
-                        "extended_timeout": "15m",
-                        "interval": "2m",
-                        "checker": "fping -q -- %%(host)s",
-                        "host": "",
-                        "approval_delay": "0s",
-                        "approval_duration": "1s",
-                        }
-    client_config = configparser.SafeConfigParser(client_defaults)
+    client_config = configparser.SafeConfigParser(Client.client_defaults)
     client_config.read(os.path.join(server_settings["configdir"],
                                     "clients.conf"))
     
@@ -2179,31 +2200,9 @@
         client_class = functools.partial(ClientDBusTransitional,
                                          bus = bus)
     
-    special_settings = {
-        # Some settings need to be accessd by special methods;
-        # booleans need .getboolean(), etc.  Here is a list of them:
-        "approved_by_default":
-            lambda section:
-            client_config.getboolean(section, "approved_by_default"),
-        "enabled":
-            lambda section:
-            client_config.getboolean(section, "enabled"),
-        }
-    # Construct a new dict of client settings of this form:
-    # { client_name: {setting_name: value, ...}, ...}
-    # with exceptions for any special settings as defined above
-    client_settings = dict((clientname,
-                           dict((setting,
-                                 (value
-                                  if setting not in special_settings
-                                  else special_settings[setting]
-                                  (clientname)))
-                                for setting, value in
-                                client_config.items(clientname)))
-                          for clientname in client_config.sections())
-    
+    client_settings = Client.config_parser(client_config)
     old_client_settings = {}
-    clients_data = []
+    clients_data = {}
     
     # Get client data and settings from last running state.
     if server_settings["restore"]:
@@ -2218,10 +2217,8 @@
             if e.errno != errno.ENOENT:
                 raise
     
-    with Crypto() as crypt:
-        for client in clients_data:
-            client_name = client["name"]
-            
+    with PGPEngine() as pgp:
+        for client_name, client in clients_data.iteritems():
             # Decide which value to use after restoring saved state.
             # We have three different values: Old config file,
             # new config file, and saved state.
@@ -2235,73 +2232,57 @@
                     if (name != "secret" and
                         value != old_client_settings[client_name]
                         [name]):
-                        setattr(client, name, value)
+                        client[name] = value
                 except KeyError:
                     pass
             
             # Clients who has passed its expire date can still be
-            # enabled if its last checker was sucessful.  Clients
+            # enabled if its last checker was successful.  Clients
             # whose checker failed before we stored its state is
             # assumed to have failed all checkers during downtime.
-            if client["enabled"] and client["last_checked_ok"]:
-                if ((datetime.datetime.utcnow()
-                     - client["last_checked_ok"])
-                    > client["interval"]):
-                    if client["last_checker_status"] != 0:
+            if client["enabled"]:
+                if datetime.datetime.utcnow() >= client["expires"]:
+                    if not client["last_checked_ok"]:
+                        logger.warning(
+                            "disabling client {0} - Client never "
+                            "performed a successfull checker"
+                            .format(client["name"]))
+                        client["enabled"] = False
+                    elif client["last_checker_status"] != 0:
+                        logger.warning(
+                            "disabling client {0} - Client "
+                            "last checker failed with error code {1}"
+                            .format(client["name"],
+                                    client["last_checker_status"]))
                         client["enabled"] = False
                     else:
                         client["expires"] = (datetime.datetime
                                              .utcnow()
                                              + client["timeout"])
-            
-            client["changedstate"] = (multiprocessing_manager
-                                      .Condition
-                                      (multiprocessing_manager
-                                       .Lock()))
-            if use_dbus:
-                new_client = (ClientDBusTransitional.__new__
-                              (ClientDBusTransitional))
-                tcp_server.clients[client_name] = new_client
-                new_client.bus = bus
-                for name, value in client.iteritems():
-                    setattr(new_client, name, value)
-                client_object_name = unicode(client_name).translate(
-                    {ord("."): ord("_"),
-                     ord("-"): ord("_")})
-                new_client.dbus_object_path = (dbus.ObjectPath
-                                               ("/clients/"
-                                                + client_object_name))
-                DBusObjectWithProperties.__init__(new_client,
-                                                  new_client.bus,
-                                                  new_client
-                                                  .dbus_object_path)
-            else:
-                tcp_server.clients[client_name] = (Client.__new__
-                                                   (Client))
-                for name, value in client.iteritems():
-                    setattr(tcp_server.clients[client_name],
-                            name, value)
-            
+                    
             try:
-                tcp_server.clients[client_name].secret = (
-                    crypt.decrypt(tcp_server.clients[client_name]
-                                  .encrypted_secret,
-                                  client_settings[client_name]
-                                  ["secret"]))
-            except CryptoError:
+                client["secret"] = (
+                    pgp.decrypt(client["encrypted_secret"],
+                                client_settings[client_name]
+                                ["secret"]))
+            except PGPError:
                 # If decryption fails, we use secret from new settings
-                tcp_server.clients[client_name].secret = (
+                logger.debug("Failed to decrypt {0} old secret"
+                             .format(client_name))
+                client["secret"] = (
                     client_settings[client_name]["secret"])
+
     
-    # Create/remove clients based on new changes made to config
-    for clientname in set(old_client_settings) - set(client_settings):
-        del tcp_server.clients[clientname]
-    for clientname in set(client_settings) - set(old_client_settings):
-        tcp_server.clients[clientname] = (client_class(name
-                                                       = clientname,
-                                                       config =
-                                                       client_settings
-                                                       [clientname]))
+    # Add/remove clients based on new changes made to config
+    for client_name in set(old_client_settings) - set(client_settings):
+        del clients_data[client_name]
+    for client_name in set(client_settings) - set(old_client_settings):
+        clients_data[client_name] = client_settings[client_name]
+
+    # Create clients all clients
+    for client_name, client in clients_data.iteritems():
+        tcp_server.clients[client_name] = client_class(
+            name = client_name, settings = client)
     
     if not tcp_server.clients:
         logger.warning("No clients defined")
@@ -2319,7 +2300,6 @@
             # "pidfile" was never created
             pass
         del pidfilename
-        
         signal.signal(signal.SIGINT, signal.SIG_IGN)
     
     signal.signal(signal.SIGHUP, lambda signum, frame: sys.exit())
@@ -2394,17 +2374,18 @@
         # Store client before exiting. Secrets are encrypted with key
         # based on what config file has. If config file is
         # removed/edited, old secret will thus be unrecovable.
-        clients = []
-        with Crypto() as crypt:
+        clients = {}
+        with PGPEngine() as pgp:
             for client in tcp_server.clients.itervalues():
                 key = client_settings[client.name]["secret"]
-                client.encrypted_secret = crypt.encrypt(client.secret,
-                                                        key)
+                client.encrypted_secret = pgp.encrypt(client.secret,
+                                                      key)
                 client_dict = {}
                 
-                # A list of attributes that will not be stored when
-                # shutting down.
-                exclude = set(("bus", "changedstate", "secret"))
+                # A list of attributes that can not be pickled
+                # + secret.
+                exclude = set(("bus", "changedstate", "secret",
+                               "checker"))
                 for name, typ in (inspect.getmembers
                                   (dbus.service.Object)):
                     exclude.add(name)
@@ -2415,7 +2396,7 @@
                     if attr not in exclude:
                         client_dict[attr] = getattr(client, attr)
                 
-                clients.append(client_dict)
+                clients[client.name] = client_dict
                 del client_settings[client.name]["secret"]
         
         try:
@@ -2497,6 +2478,5 @@
     # Must run before the D-Bus bus name gets deregistered
     cleanup()
 
-
 if __name__ == '__main__':
     main()

=== modified file 'mandos-ctl'
--- mandos-ctl	2011-10-15 16:48:03 +0000
+++ mandos-ctl	2011-12-21 00:29:33 +0000
@@ -93,29 +93,23 @@
     >>> string_to_delta("5m 30s")
     datetime.timedelta(0, 330)
     """
-    timevalue = datetime.timedelta(0)
-    regexp = re.compile("\d+[dsmhw]")
+    value = datetime.timedelta(0)
+    regexp = re.compile("(\d+)([dsmhw]?)")
     
-    for s in regexp.findall(interval):
-        try:
-            suffix = unicode(s[-1])
-            value = int(s[:-1])
-            if suffix == "d":
-                delta = datetime.timedelta(value)
-            elif suffix == "s":
-                delta = datetime.timedelta(0, value)
-            elif suffix == "m":
-                delta = datetime.timedelta(0, 0, 0, 0, value)
-            elif suffix == "h":
-                delta = datetime.timedelta(0, 0, 0, 0, 0, value)
-            elif suffix == "w":
-                delta = datetime.timedelta(0, 0, 0, 0, 0, 0, value)
-            else:
-                raise ValueError
-        except (ValueError, IndexError):
-            raise ValueError
-        timevalue += delta
-    return timevalue
+    for num, suffix in regexp.findall(interval):
+        if suffix == "d":
+            value += datetime.timedelta(int(num))
+        elif suffix == "s":
+            value += datetime.timedelta(0, int(num))
+        elif suffix == "m":
+            value += datetime.timedelta(0, 0, 0, 0, int(num))
+        elif suffix == "h":
+            value += datetime.timedelta(0, 0, 0, 0, 0, int(num))
+        elif suffix == "w":
+            value += datetime.timedelta(0, 0, 0, 0, 0, 0, int(num))
+        elif suffix == "":
+            value += datetime.timedelta(0, 0, 0, int(num))
+    return value
 
 def print_clients(clients, keywords):
     def valuetostring(value, keyword):

=== modified file 'mandos-keygen.xml'
--- mandos-keygen.xml	2011-10-05 16:00:56 +0000
+++ mandos-keygen.xml	2011-11-12 18:14:55 +0000
@@ -214,7 +214,7 @@
 	<listitem>
 	  <para>
 	    Target directory for key files.  Default is
-	    <filename>/etc/mandos</filename>.
+	    <filename class="directory">/etc/mandos</filename>.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -411,7 +411,7 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename>/tmp</filename></term>
+	<term><filename class="directory">/tmp</filename></term>
 	<listitem>
 	  <para>
 	    Temporary files will be written here if
@@ -452,9 +452,9 @@
     </informalexample>
     <informalexample>
       <para>
-	Prompt for a password, encrypt it with the key in
-	<filename>/etc/mandos</filename> and output a section suitable
-	for <filename>clients.conf</filename>.
+	Prompt for a password, encrypt it with the key in <filename
+	class="directory">/etc/mandos</filename> and output a section
+	suitable for <filename>clients.conf</filename>.
       </para>
       <para>
 	<userinput>&COMMANDNAME; --password</userinput>

=== modified file 'mandos.xml'
--- mandos.xml	2011-11-26 22:22:20 +0000
+++ mandos.xml	2011-12-21 00:29:33 +0000
@@ -407,7 +407,7 @@
     <title>LOGGING</title>
     <para>
       The server will send log message with various severity levels to
-      <filename>/dev/log</filename>.  With the
+      <filename class="devicefile">/dev/log</filename>.  With the
       <option>--debug</option> option, it will log even more messages,
       and also show them on the console.
     </para>
@@ -489,6 +489,9 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
+	<term><filename class="devicefile">/dev/log</filename></term>
+      </varlistentry>
+      <varlistentry>
 	<term><filename
 	class="directory">/var/lib/mandos</filename></term>
 	<listitem>

=== modified file 'plugins.d/plymouth.xml'
--- plugins.d/plymouth.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/plymouth.xml	2011-11-12 18:14:55 +0000
@@ -161,7 +161,7 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename>/proc</filename></term>
+	<term><filename class="directory">/proc</filename></term>
 	<listitem>
 	  <para>
 	    To find the running <citerefentry><refentrytitle

=== modified file 'plugins.d/usplash.xml'
--- plugins.d/usplash.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/usplash.xml	2011-11-12 18:14:55 +0000
@@ -178,7 +178,7 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename>/proc</filename></term>
+	<term><filename class="directory">/proc</filename></term>
 	<listitem>
 	  <para>
 	    To find the running <citerefentry><refentrytitle

